﻿using System.Drawing;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace GridAndRepeater
{
    public interface INestedGridElementDataSourceProvider
    {
        object GetBindingSource(object currentRow, object keyValue);
    }

    public class NestedComboCellElement : GridDataCellElement
    {
        private readonly INestedGridElementDataSourceProvider dataSourceProvider;

        private RadComboBoxElement combo;

        public NestedComboCellElement(INestedGridElementDataSourceProvider dataSourceProvider, GridViewColumn column, GridRowElement row)
            : base(column, row)
        {
            this.dataSourceProvider = dataSourceProvider;

            this.Padding = new Padding(2);
        }
        
        protected override void SetContentCore(object value)
        {
            this.EnsureChildElements();

            combo.DataSource = dataSourceProvider.GetBindingSource(base.RowInfo.DataBoundItem, value);
            combo.DisplayMember = "OrderID";
            combo.ValueMember = "OrderID";
        }

        //flag needed to avoid loops when focus changes from grid to hosted control
        private bool currentCellChanging = false;

        protected override void CreateChildElements()
        {
            base.CreateChildElements();

            combo = new RadComboBoxElement();

            combo.Alignment = ContentAlignment.MiddleCenter;
            combo.StretchVertically = false;
            combo.Margin = new Padding(5);

            combo.TextBoxElement.TextBoxItem.GotFocus +=
                delegate
                    {
                        if (!currentCellChanging &&
                            !this.RowElement.RowInfo.IsCurrent)
                        {
                            this.GridControl.EndEdit();
                            this.RowElement.RowInfo.IsCurrent = true;
                            this.IsCurrent = true;
                        }
                        currentCellChanging = false;
                    };            

            this.Children.Add(combo);
        }

        public override void OnTunnelEvent(RadElement sender, RoutedEventArgs args)
        {
            if (args.RoutedEvent == RadElement.ParentChangedEvent)
            {
                if (this.GridControl != null)
                {
                    this.GridControl.CurrentCellChanged -=
                        new CurrentCellChangedEventHandler(GridControl_CurrentCellChanged);

                    this.GridControl.CurrentCellChanged +=
                        new CurrentCellChangedEventHandler(GridControl_CurrentCellChanged);
                }                
            }
        }

        void GridControl_CurrentCellChanged(object sender, CurrentCellChangedEventArgs e)
        {
            if (e.NewCell == null)
            {
                return;
            }

            if (this == e.NewCell)
            {
                //called after "tab" key was pressed or cell was clicked and this cell becomes current cell

                currentCellChanging = true;
                //async call needed to allow hosted control get focus after click in the grid cell, which
                //by default focuses the grid
                this.GridControl.BeginInvoke(
                    new MethodInvoker(
                        () => combo.TextBoxElement.TextBoxItem.HostedControl.Focus()));     
            }
            else if (!(e.NewCell is NestedComboCellElement) && 
                !this.GridControl.Focused)
            {
                //called after "tab" key was pressed in the combo
                e.NewCell.Focus();
            }
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);

            if (this.GridControl != null)
            {
                this.GridControl.CurrentCellChanged -= new CurrentCellChangedEventHandler(GridControl_CurrentCellChanged);
            }
        }

        protected override void OnPropertyChanged(RadPropertyChangedEventArgs e)
        {
            if (e.Property == IsFocusedProperty && !(bool)e.NewValue)
            {
                this.ElementTree.Control.Focus();
            }
        }
    }
}
